/* ----------------------- System includes ----------------------------------*/
#include "stdlib.h"
#include "string.h"

#include "COM3RS485.h"	
#include "USART2.h"
/* ----------------------- Platform includes --------------------------------*/
#include "port.h"

/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mbrtu.h"
#include "mbframe.h"

#include "mbcrc.h"
#include "mbport.h"

/* ----------------------- Defines ------------------------------------------*/
#define MB_SER_PDU_SIZE_MIN     4       /*!< Minimum size of a Modbus RTU frame. */
#define MB_SER_PDU_SIZE_MAX     256     /*!< Maximum size of a Modbus RTU frame. */
#define MB_SER_PDU_SIZE_CRC     2       /*!< Size of CRC field in PDU. */
#define MB_SER_PDU_ADDR_OFF     0       /*!< Offset of slave address in Ser-PDU. */
#define MB_SER_PDU_PDU_OFF      1       /*!< Offset of Modbus-PDU in Ser-PDU. */

// #define	DEBUG

/* ----------------------- Type definitions ---------------------------------*/
typedef enum
{
    STATE_RX_INIT,              /*!< Receiver is in initial state. */
    STATE_RX_IDLE,              /*!< Receiver is in idle state. */
    STATE_RX_RCV,               /*!< Frame is beeing received. */
    STATE_RX_ERROR              /*!< If the frame is invalid. */
} eMBRcvState;

typedef enum
{
    STATE_TX_IDLE,              /*!< Transmitter is in idle state. */
    STATE_TX_XMIT               /*!< Transmitter is in transfer state. */
} eMBSndState;

/* ----------------------- Static variables ---------------------------------*/
static volatile eMBSndState eSndState;
static volatile eMBRcvState eRcvState;

volatile UCHAR  ucRTUBuf[MB_SER_PDU_SIZE_MAX];

static volatile UCHAR *pucSndBufferCur;
static volatile USHORT usSndBufferCount;

static volatile USHORT usRcvBufferPos;

/* ----------------------- Start implementation -----------------------------*/
eMBErrorCode eMBRTUInit( UCHAR ucSlaveAddress, UCHAR ucPort, ULONG ulBaudRate, eMBParity eParity )
{
    eMBErrorCode    eStatus = MB_ENOERR;
    ULONG           usTimerT35_50us;

    ( void )ucSlaveAddress;
    ENTER_CRITICAL_SECTION(  );

    /* Modbus RTU uses 8 Databits. */
    if( xMBPortSerialInit( ucPort, ulBaudRate, 8, eParity ) != TRUE )//ڳʼ
    {
        eStatus = MB_EPORTERR;
    }
    else
    {
        /* If baudrate > 19200 then we should use the fixed timer values
         * t35 = 1750us. Otherwise t35 must be 3.5 times the character time.
         */
        if( ulBaudRate > 19200 )
        {
            usTimerT35_50us = 35;       /* 1800us. */
        }
        else
        {
            /* The timer reload value for a character is given by:
             *
             * ChTimeValue = Ticks_per_1s / ( Baudrate / 11 )
             *             = 11 * Ticks_per_1s / Baudrate
             *             = 220000 / Baudrate
             * The reload for t3.5 is 1.5 times this value and similary
             * for t3.5.
             */
            usTimerT35_50us = ( 7UL * 220000UL ) / ( 2UL * ulBaudRate );
        }
        if( xMBPortTimersInit( ( USHORT ) usTimerT35_50us ) != TRUE )//ʱʼ
        {
            eStatus = MB_EPORTERR;
        }
    }
    EXIT_CRITICAL_SECTION(  );

    return eStatus;
}

/* 
*1:ý״̬eRcvStateΪSTATE_RX_INIT 
*2:ʹܴڽ,ֹڷ,Ϊӻ,ȴ͵; 
*3:ʱ3.5Tʱʱһж,ʱeRcvStateΪSTATE_RX_INIT,ϱʼ¼,ȻeRcvStateΪSTATE_RX_IDLE; 
*4:ÿν3.5Tʱж,ʱֹȴֽڽպ󣬲ʹܶʱ; 
*/
void eMBRTUStart( void )
{
    ENTER_CRITICAL_SECTION(  );
    /*״̬StRexRxInС
    *ʱt3.5ûнյַôǽתΪStesteRxyIDLE
    *ȷӳModbusЭջֱ߿Ϊֹ
    */
    eRcvState = STATE_RX_INIT;//״̬ö֮ʼ
    vMBPortSerialEnable( TRUE, FALSE );//ȼUSART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
    vMBPortTimersEnable(  );//ʹʱ

    EXIT_CRITICAL_SECTION(  );
}

void eMBRTUStop( void )
{	
    ENTER_CRITICAL_SECTION(  );
    vMBPortSerialEnable( FALSE, FALSE );
    vMBPortTimersDisable(  );
    EXIT_CRITICAL_SECTION(  );
}

/*eMBPollѯEV_FRAME_RECEIVED¼ʱ,peMBFrameReceiveCur()˺ûΪָpeMBFrameReceiveCur()ĸֵ 
*˺ɵĹܣһ֡ݱУȡmodbusӻַpucRcvAddressPDUĵĳȸpusLengthPDUĵ׵ַpucFrame 
*βȫΪַ*/ 
eMBErrorCode eMBRTUReceive( UCHAR * pucRcvAddress, UCHAR ** pucFrame, USHORT * pusLength )
{
    BOOL            xFrameReceived = FALSE;
    eMBErrorCode    eStatus = MB_ENOERR;
	
    ENTER_CRITICAL_SECTION(  );
    assert( usRcvBufferPos < MB_SER_PDU_SIZE_MAX );/*Ժ꣬жϽյֽ<256>256ֹ*/  

    /* Length and CRC check */
    if( ( usRcvBufferPos >= MB_SER_PDU_SIZE_MIN ) && ( usMBCRC16( ( UCHAR * ) ucRTUBuf, usRcvBufferPos ) == 0 ) )
    {
        /* Save the address field. All frames are passed to the upper layed
         * and the decision if a frame is used is done there.
         */
        *pucRcvAddress = ucRTUBuf[MB_SER_PDU_ADDR_OFF];//ȡյĵһֽڣmodbusӻַ  

        /* Total length of Modbus-PDU is Modbus-Serial-Line-PDU minus
         * size of address field and CRC checksum.
         */
        *pusLength = ( USHORT )( usRcvBufferPos - MB_SER_PDU_PDU_OFF - MB_SER_PDU_SIZE_CRC );//3  

        /* Return the start of the Modbus PDU to the caller. */
        *pucFrame = ( UCHAR * ) & ucRTUBuf[MB_SER_PDU_PDU_OFF];
        xFrameReceived = TRUE;
        xFrameReceived = xFrameReceived;
    }
    else
    {
        eStatus = MB_EIO;
    }

    EXIT_CRITICAL_SECTION(  );
    return eStatus;
}

/* 
*1:ӦPDUǰϴӻַ; 
*2:ӦPDUCRCУ; 
*3:ʹܷͣ; 
*/  
eMBErrorCode eMBRTUSend( UCHAR ucSlaveAddress, const UCHAR * pucFrame, USHORT usLength )
{
    eMBErrorCode    eStatus = MB_ENOERR;
    USHORT          usCRC16;
	
    ENTER_CRITICAL_SECTION(  );
    /*
    *ջǷڿ״̬
    *ǿУﴦյ֡ˣͬʱһ֡ǲò֡
    */
    if( eRcvState == STATE_RX_IDLE )
    {
        /* First byte before the Modbus-PDU is the slave address. */
         /*ЭݵԪǰӴӻַ*/  
        pucSndBufferCur = ( UCHAR * ) pucFrame - 1;
        usSndBufferCount = 1;

        /* Now copy the Modbus-PDU into the Modbus-Serial-Line-PDU. */
        pucSndBufferCur[MB_SER_PDU_ADDR_OFF] = ucSlaveAddress;
        usSndBufferCount += usLength;

        /* Calculate CRC16 checksum for Modbus-Serial-Line-PDU. */
        usCRC16 = usMBCRC16( ( UCHAR * ) pucSndBufferCur, usSndBufferCount );
        ucRTUBuf[usSndBufferCount++] = ( UCHAR )( usCRC16 & 0xFF );
        ucRTUBuf[usSndBufferCount++] = ( UCHAR )( usCRC16 >> 8 );

        /* Activate the transmitter. */
        eSndState = STATE_TX_XMIT;//״̬
        
        xMBPortSerialPutByte((CHAR)*pucSndBufferCur);/*һֽڵݣ뷢жϺ*/  
        pucSndBufferCur++;
        usSndBufferCount--;
        vMBPortSerialEnable( FALSE, TRUE );/*ʹֹܷͣ*/
    }
    else
    {
        eStatus = MB_EIO;
    }
    EXIT_CRITICAL_SECTION(  );
    return eStatus;
}

/*  
*1:յݴucRTUBuf[];  
*2:usRcvBufferPosΪȫֱʾݵĸ;    
*3:ÿյһֽڵݣ3.5Tʱ0  
*/ 
BOOL xMBRTUReceiveFSM( void )//////////////////////////////////////////////////////////////////////////////////////////////////жϺʵеĺ
{
    BOOL            xTaskNeedSwitch = FALSE;
    UCHAR           ucByte;

    assert( eSndState == STATE_TX_IDLE );/*ȷûڷ*/	
	
    /* Always read the character. */
    ( void )xMBPortSerialGetByte( ( CHAR * ) & ucByte );/*ӴݼĴȡһֽ*/

    //ݲͬ״̬ת
    switch ( eRcvState )
    {
        /* If we have received a character in the init state we have to
         * wait until the frame is finished.
         */
    case STATE_RX_INIT:
        vMBPortTimersEnable( );/*3.5Tʱ*/ 
        break;

        /* In the error state we wait until all characters in the
         * damaged frame are transmitted.
         */
    case STATE_RX_ERROR:/*֡𻵣ʱ洮ڽյ*/  
        vMBPortTimersEnable( );
        break;

        /* In the idle state we wait for a new character. If a character
         * is received the t1.5 and t3.5 timers are started and the
         * receiver is in the state STATE_RX_RECEIVCE.
         */
    case STATE_RX_IDLE: /*УʼգSTATE_RX_RCV״̬*/  
        usRcvBufferPos = 0;
        ucRTUBuf[usRcvBufferPos++] = ucByte; /**/
        eRcvState = STATE_RX_RCV;

        /* Enable t3.5 timers. */
        vMBPortTimersEnable( ); /*ÿյһֽڣ3.5Tʱ*/ 
        break;

        /* We are currently receiving a frame. Reset the timer after
         * every character received. If more than the maximum possible
         * number of bytes in a modbus frame is received the frame is
         * ignored.
         */
    case STATE_RX_RCV:
        if( usRcvBufferPos < MB_SER_PDU_SIZE_MAX )
        {
            ucRTUBuf[usRcvBufferPos++] = ucByte;/**/
        }
        else
        {
            eRcvState = STATE_RX_ERROR;/*һ֡ĵֽPDUȣԳ*/  
        }
        vMBPortTimersEnable();/*ÿյһֽڣ3.5Tʱ*/ 
        break;
    }
		
    return xTaskNeedSwitch;
}

/*뷢жϣڷжеprvvUARTTxReadyISR()
pxMBFrameCBTransmitterEmpty()
pxMBFrameCBTransmitterEmptyΪָ룬յxMBRTUTransmitFSM()
*/
BOOL xMBRTUTransmitFSM( void )//////////////////////////////////////////////////////////////////////////////////////////жʵеĺ
{
    BOOL            xNeedPoll = FALSE;

    assert( eRcvState == STATE_RX_IDLE );
	
    switch ( eSndState )
    {
        /* We should not get a transmitter event if the transmitter is in
         * idle state.  */
    case STATE_TX_IDLE:/*ڿ״̬ʹܽգֹ*/  
        /* enable receiver/disable transmitter. */
        vMBPortSerialEnable( TRUE, FALSE );
        break;

    case STATE_TX_XMIT:/*ڷ״̬,ڴӻͺeMBRTUSendиֵSTATE_TX_XMIT*/
        /* check if we are finished. */
        if( usSndBufferCount != 0 )
        {
            xMBPortSerialPutByte( ( CHAR )*pucSndBufferCur );
            pucSndBufferCur++;  /* next byte in sendbuffer. */
            usSndBufferCount--;
        }
        else
        {
            do{}while(USART_GetITStatus(USART2, USART_IT_TC) == SET);
            //񣬷
            xNeedPoll = xMBPortEventPost( EV_FRAME_SENT ); /*Эջ¼״ֵ̬ΪEV_FRAME_SENT,¼,eMBPollԴ¼д*/  
            /* Disable transmitter. This prevents another transmit buffer
             * empty interrupt. */
            vMBPortSerialEnable( TRUE, FALSE ); /*ʹܽգֹ*/
            eSndState = STATE_TX_IDLE;/*״̬Ϊ״̬*/  
        }
        break;
    }

    return xNeedPoll;
}

/* 
*1:ӻһ֡ݺ󣬽״̬eRcvStateΪSTATE_RX_RCV 
*2:ϱյġ¼(EV_FRAME_RECEIVED) 
*3:ֹ3.5Tʱý״̬eRcvState״̬ΪSTATE_RX_IDLE; 
*/  
BOOL xMBRTUTimerT35Expired( void )//3.5T
{
    BOOL            xNeedPoll = FALSE;
	
    switch ( eRcvState )
    {
        /* Timer t35 expired. Startup phase is finished. */
        /*ϱmodbusЭջ¼״̬poll,EV_READY:ʼ¼*/
        case STATE_RX_INIT:
            xNeedPoll = xMBPortEventPost( EV_READY );
            break;

            /* A frame was received and t35 expired. Notify the listener that
             * a new frame was received. */
        case STATE_RX_RCV: /*һ֡ݽ*/
            xNeedPoll = xMBPortEventPost( EV_FRAME_RECEIVED );/*ϱЭջ¼,յһ֡*/ 
            break;

            /* An error occured while receiving the frame. */
        case STATE_RX_ERROR:
            break;

            /* Function called in an illegal state. */
        default:
            assert( ( eRcvState == STATE_RX_INIT ) ||
                    ( eRcvState == STATE_RX_RCV ) || ( eRcvState == STATE_RX_ERROR ) );
    }

    vMBPortTimersDisable(  );/*յһ֡ݺ󣬽ֹ3.5Tʱֻһ֡ݿʼʼʱ*/   
    eRcvState = STATE_RX_IDLE;/*һ֡ݣ״̬Ϊ*/

    return xNeedPoll;
}
